-- $Id: PerfQueries.sql,v 1.1 2006/07/19 06:46:24 pepm01a Exp $
--
-- This script expects to be run from SQSH.
-- Use the \set command to define the variables before running it:
-- \set srv=servername
-- \set date=startdate

set nocount on

declare @srv varchar(32), @dt date

select @srv="$srv", @dt='20060101'

if "$date" <> ""
    select @dt="$date"

print 'Performance Stats for %1! since %2!', @srv, @dt
print 'Weekdays, between 8AM and 6PM'

--print "CPU (secondes)"

select dt=convert(date, runDate), CPU=sum(UserCPUTime+SystemCPUTime) 
  into #cpu 
  from monEngine 
 where servername=@srv and CPUTime > 0
   and runDate > @dt
   and convert(time, runDate) between '08:00' and '18:00'
--   and convert(date, runDate) != 'feb 1 2006'
   and datepart(dw, runDate) in (2,3,4,5,6)
 group by convert(date, runDate) order by 1

--print "IO Physiques (en milliers)"
select dt=convert(date, runDate), Reads=sum(Reads+APFReads), Writes=sum(Writes) 
  into #disk
  from monDeviceIO
 where servername=@srv 
   and runDate > @dt
   and convert(time, runDate) between '08:00' and '18:00'
--   and convert(date, runDate) != 'feb 1 2006'
   and datepart(dw, runDate) in (2,3,4,5,6)
 group by convert(date, runDate) order by 1

--print "Waits (WaitTime = dure des attentes, Waits = nombre d'occurences)"
select dt=convert(date, runDate), WaitTime=sum(WaitTime)/1000, Waits=sum(Waits)
 into #wait
 from monProcessWaits
 where servername=@srv and WaitEventID not in (250, 260, 221, 222)
   and runDate > @dt
   and convert(time, runDate) between '08:00' and '18:00'
--  and convert(date, runDate) != 'feb 1 2006'
   and datepart(dw, runDate) in (2,3,4,5,6)
 group by convert(date, runDate) order by 1

--print "IO physique vs. logique (en millier)"
select dt=convert(date, runDate), PagesRead=sum(PagesRead * 1.0), Physical=sum(PhysicalReads * 1.0), Logical=sum(LogicalReads * 1.0), Transactions=sum(Transactions * 1.0), Writes=sum(PagesWritten * 1.0), CPU=sum(CPUTime/1000.0)
  into #phys
  from monProcessActivity where servername=@srv 
   and convert(time, runDate) between '08:00' and '18:00'
   and runDate > @dt
--   and convert(date, runDate) != 'feb 1 2006'
   and datepart(dw, runDate) in (2,3,4,5,6)
--   and isnull(Login, 'sa') not in ('sa')
 group by convert(date, runDate) 
--having sum(Waits) > 1000000 
 order by 1

--select * from #phys

select Date=convert(varchar(8), c.dt, 112)
     , CPU=str(c.CPU, 6, 0)
     , "Rds K" = str(d.Reads/1000, 5, 0)
     , "Wrt K" = str(d.Writes/1000, 6, 0)
     , 'Waits K' = str(w.Waits / 1000, 8, 0)
     , 'WaitTime' = str(w.WaitTime, 8, 0)
     , 'PhysRead K' = str(p.Physical/1000, 8, 2)
     , 'PgsRead K' = str(p.PagesRead/1000, 8, 2)
     , 'LogRead K' = str(p.Logical/1000, 8, 2)
     , 'UWrites K' = str(p.Writes/1000, 8, 2)
     , CHR=str(convert(numeric(5,2), (1-convert(float, p.PagesRead)/p.Logical) * 100), 5, 2)
     , 'Trans K' = str(p.Transactions/1000, 8, 2)
     , 'UCPU' = str(p.CPU, 5, 0)
--     , 'UCPU' = p.CPU
  from #cpu c
     , #disk d
     , #wait w
     , #phys p
 where w.dt = c.dt
   and d.dt = c.dt
   and p.dt = c.dt
 order by 1


drop table #cpu
drop table #disk
drop table #wait
drop table #phys

set nocount off

go
