/* $Id: collect.sql,v 1.1 2006/07/06 07:56:02 pepm01a Exp $ */

use mondb
go

-- installation/setup

if object_id('sp__mda_init') != NULL
    drop proc sp__mda_init
go
if object_id('sp__mda_collect') != NULL
    drop proc sp__mda_collect
go
if object_id('sp__mda_run_collect') != NULL
    drop proc sp__mda_run_collect
go


create proc sp__mda_init
as
declare @sql varchar(1024)

if object_id('mondb..monEngine') = NULL
begin
    select @sql = 'create table mondb..monEngine(EngineNumber int, CPUTime int, SystemCPUTime int, UserCPUTime int, IdleCPUTime int, ContextSwitches int,
            runDate smalldatetime default getdate(), servername varchar(30) default "' 
    + @@servername + '")'
    exec(@sql)
end

if object_id('mondb..monProcessWaits') = NULL
begin
    select @sql = 'create table mondb..monProcessWaits(SPID smallint, KPID int, WaitEventID int, WaitTime int, Waits int, 
                runDate smalldatetime default getdate(), servername varchar(30) default "' + @@servername + '")'
    exec(@sql)
end

if object_id('mondb..monDeviceIO')= NULL
begin
    select @sql = 'create table mondb..monDeviceIO(Reads int, APFReads int, Writes int, DevSemaphoreRequests int, DevSemaphoreWaits int,
                 IOTime int, LogicalName varchar(60), PhysicalName varchar(255),
                runDate smalldatetime default getdate(), servername varchar(30) default "' + @@servername + '")'
    exec(@sql)
end

if object_id('mondb..monNetworkIO') = NULL
begin
    select @sql = 'create table mondb..monNetworkIO(PacketsSent int, PacketsReceived int, BytesSent int, BytesReceived int,
                runDate smalldatetime default getdate(), servername varchar(30) default "' + @@servername + '")'
    exec(@sql)
end

if object_id('mondb..monProcessActivity') = NULL
begin
    select @sql = 'create table mondb..monProcessActivity(
 SPID            smallint,
 KPID            int    ,
 CPUTime         int    ,
 WaitTime        int    ,
 PhysicalReads   int    ,
 LogicalReads    int    ,
 PagesRead       int    ,
 PhysicalWrites  int    ,
 PagesWritten    int    ,
 MemUsageKB      int    ,
 LocksHeld       int    ,
 TableAccesses   int    ,
 IndexAccesses   int    ,
 TempDbObjects   int    ,
 ULCBytesWritten int    ,
 ULCFlushes      int    ,
 ULCFlushFull    int    ,
 ULCMaxUsage     int    ,
 ULCCurrentUsage int    ,
 Transactions    int    ,
 Commits         int    ,
 Rollbacks       int    ,
 Login           varchar(30) null,
 Application     varchar(30) null,
 ClientIP        varchar(24) null,
 runDate         smalldatetime default getdate(),
 servername      varchar(30) default "' + @@servername + '")'
    exec(@sql)
end

if object_id('mondb..monOpenObjectActivity') = NULL
begin
    select @sql = 'create table mondb..monOpenObjectActivity(
 DBID            int    ,
 ObjectID        int    ,
 LogicalReads    int    ,
 PhysicalReads   int    ,
 APFReads   int    ,
 PagesRead    int    ,
 PhysicalWrites  int    ,
 PagesWritten    int    ,
 UsedCount      int    ,
 RowsInserted   int    ,
 RowsDeleted    int    ,
 RowsUpdated   int    ,
 Operations   int    ,
 LockRequests int    ,
 LockWaits    int ,
 runDate         smalldatetime default getdate(),
 servername      varchar(30) default "' + @@servername + '")'
    exec(@sql)
end

go

exec sp__mda_init
go


create proc sp__mda_run_collect
(@delay         varchar(20))
as

-- Initialization.
    create table #monEngine1(EngineNumber int, CPUTime int, SystemCPUTime int, 
                             UserCPUTime int, IdleCPUTime int, 
                             ContextSwitches int)
    create table #monEngine2(EngineNumber int, CPUTime int, SystemCPUTime int, 
                             UserCPUTime int, IdleCPUTime int, 
                             ContextSwitches int)
    create table #monNetIO1(PacketsSent int, PacketsReceived int, 
                            BytesSent int, BytesReceived int)
    create table #monNetIO2(PacketsSent int, PacketsReceived int, 
                            BytesSent int, BytesReceived int)
    create table #monDeviceIO1(Reads int, APFReads int, Writes int, 
                               DevSemaphoreRequests int, DevSemaphoreWaits int,
                               IOTime int, LogicalName varchar(60), 
                               PhysicalName varchar(255))
    create table #monDeviceIO2(Reads int, APFReads int, Writes int, 
                               DevSemaphoreRequests int, DevSemaphoreWaits int,
                               IOTime int, LogicalName varchar(60), 
                               PhysicalName varchar(255))
    create table #monProcWait1(SPID smallint, KPID int, 
                               WaitEventID int, WaitTime int, Waits int)
    create table #monProcWait2(SPID smallint, KPID int,
                               WaitEventID int, WaitTime int, Waits int)

    create table #monProcAct1(
              SPID            smallint,
              KPID            int    ,
              CPUTime         int    ,
              WaitTime        int    ,
              PhysicalReads   int    ,
              LogicalReads    int    ,
              PagesRead       int    ,
              PhysicalWrites  int    ,
              PagesWritten    int    ,
              MemUsageKB      int    ,
              LocksHeld       int    ,
              TableAccesses   int    ,
              IndexAccesses   int    ,
              TempDbObjects   int    ,
              ULCBytesWritten int    ,
              ULCFlushes      int    ,
              ULCFlushFull    int    ,
              ULCMaxUsage     int    ,
              ULCCurrentUsage int    ,
              Transactions    int    ,
              Commits         int    ,
              Rollbacks       int    )
    create table #monProcAct2 (
              SPID            smallint,
              KPID            int    ,
              CPUTime         int    ,
              WaitTime        int    ,
              PhysicalReads   int    ,
              LogicalReads    int    ,
              PagesRead       int    ,
              PhysicalWrites  int    ,
              PagesWritten    int    ,
              MemUsageKB      int    ,
              LocksHeld       int    ,
              TableAccesses   int    ,
              IndexAccesses   int    ,
              TempDbObjects   int    ,
              ULCBytesWritten int    ,
              ULCFlushes      int    ,
              ULCFlushFull    int    ,
              ULCMaxUsage     int    ,
              ULCCurrentUsage int    ,
              Transactions    int    ,
              Commits         int    ,
              Rollbacks       int    )

    create table #monOpObj1 (
 DBID            int    ,
 ObjectID        int    ,
 LogicalReads    int    ,
 PhysicalReads   int    ,
 APFReads   int    ,
 PagesRead    int    ,
 PhysicalWrites  int    ,
 PagesWritten    int    ,
 UsedCount      int    ,
 RowsInserted   int    ,
 RowsDeleted    int    ,
 RowsUpdated   int    ,
 Operations   int    ,
 LockRequests int    ,
 LockWaits    int )
    create table #monOpObj2 (
 DBID            int    ,
 ObjectID        int    ,
 LogicalReads    int    ,
 PhysicalReads   int    ,
 APFReads   int    ,
 PagesRead    int    ,
 PhysicalWrites  int    ,
 PagesWritten    int    ,
 UsedCount      int    ,
 RowsInserted   int    ,
 RowsDeleted    int    ,
 RowsUpdated   int    ,
 Operations   int    ,
 LockRequests int    ,
 LockWaits    int )


    insert #monEngine1 
         select EngineNumber, CPUTime, SystemCPUTime, UserCPUTime, 
                IdleCPUTime, ContextSwitches
           from master..monEngine

    insert #monProcWait1(SPID, KPID, WaitEventID, WaitTime, Waits)
        select SPID, KPID, WaitEventID, WaitTime, Waits 
          from master..monProcessWaits

    insert #monDeviceIO1(Reads, APFReads, Writes, DevSemaphoreRequests, 
                         DevSemaphoreWaits, IOTime, LogicalName, PhysicalName)
        select Reads, APFReads, Writes, DevSemaphoreRequests, 
               DevSemaphoreWaits, IOTime, LogicalName, PhysicalName 
          from master..monDeviceIO
    insert #monNetIO1(PacketsSent, PacketsReceived, BytesSent, BytesReceived)
        select PacketsSent, PacketsReceived, BytesSent, BytesReceived 
          from master..monNetworkIO

    insert #monProcAct1(
              SPID,
              KPID,
              CPUTime,
              WaitTime,
              PhysicalReads    ,
              LogicalReads    ,
              PagesRead        ,
              PhysicalWrites    ,
              PagesWritten     ,
              MemUsageKB       ,
              LocksHeld        ,
              TableAccesses     ,
              IndexAccesses     ,
              TempDbObjects     ,
              ULCBytesWritten    ,
              ULCFlushes        ,
              ULCFlushFull      ,
              ULCMaxUsage       ,
              ULCCurrentUsage    ,
              Transactions       ,
              Commits            ,
              Rollbacks         )
        select SPID,
              KPID,
              CPUTime,
              WaitTime,
              PhysicalReads    ,
              LogicalReads    ,
              PagesRead        ,
              PhysicalWrites    ,
              PagesWritten     ,
              MemUsageKB       ,
              LocksHeld        ,
              TableAccesses     ,
              IndexAccesses     ,
              TempDbObjects     ,
              ULCBytesWritten    ,
              ULCFlushes        ,
              ULCFlushFull      ,
              ULCMaxUsage       ,
              ULCCurrentUsage    ,
              Transactions       ,
              Commits            ,
              Rollbacks 
         from master..monProcessActivity

    insert #monOpObj1 (
 DBID,
 ObjectID,
 LogicalReads,
 PhysicalReads,
 APFReads,
 PagesRead,
 PhysicalWrites,
 PagesWritten,
 UsedCount,
 RowsInserted,
 RowsDeleted,
 RowsUpdated,
 Operations,
 LockRequests,
 LockWaits )
     select 
 DBID,
 ObjectID,
 LogicalReads,
 PhysicalReads,
 APFReads,
 PagesRead,
 PhysicalWrites,
 PagesWritten,
 UsedCount,
 RowsInserted,
 RowsDeleted,
 RowsUpdated,
 Operations,
 LockRequests,
 LockWaits from master..monOpenObjectActivity where IndexID = 0


declare @start varchar(20)

declare @dt smalldatetime, @prev smalldatetime, @now smalldatetime

select @prev = getdate()

while(1 = 1)
begin

  try_again:
    select @dt = dateadd(mi, 1, @prev)
    select @start = substring(convert(varchar, @dt, 8), 1, 5) + ":00"

    select @now = getdate()

    if datediff(mi, @now, @dt) < 1
    begin
        print "%1!: start (%2!) is in the past (now: %3!)", @prev, @start, @now
        select @prev = @dt
        goto try_again
    end

    waitfor time @start

    select @prev = @dt

    -- monEngine
    insert #monEngine2 
        select EngineNumber, CPUTime, SystemCPUTime, 
               UserCPUTime, IdleCPUTime, ContextSwitches
          from master..monEngine
    insert mondb..monEngine(EngineNumber, CPUTime, SystemCPUTime, UserCPUTime,
                            IdleCPUTime, ContextSwitches, runDate)
     select e1.EngineNumber, e2.CPUTime - e1.CPUTime
          , e2.SystemCPUTime - e1.SystemCPUTime
          , e2.UserCPUTime - e1.UserCPUTime
          , e2.IdleCPUTime - e1.IdleCPUTime
          , e2.ContextSwitches - e1.ContextSwitches
          , @dt
       from #monEngine1 e1, #monEngine2 e2
      where e1.EngineNumber = e2.EngineNumber
    truncate table #monEngine1
    insert #monEngine1 
        select EngineNumber, CPUTime, SystemCPUTime, 
               UserCPUTime, IdleCPUTime, ContextSwitches
       from #monEngine2
    truncate table #monEngine2

    -- monProcessWaits
    insert #monProcWait2(SPID, KPID, WaitEventID, WaitTime, Waits)
        select SPID, KPID, WaitEventID, WaitTime, Waits 
          from master..monProcessWaits
    insert mondb..monProcessWaits(SPID, KPID, WaitEventID, WaitTime, Waits, runDate)
        select w2.SPID, w2.KPID,
               w2.WaitEventID, w2.WaitTime - w1.WaitTime, w2.Waits - w1.Waits,
               @dt
          from #monProcWait1 w1, #monProcWait2 w2
         where w1.WaitEventID = w2.WaitEventID
           and w1.SPID = w2.SPID
           and w1.KPID = w2.KPID
           and w2.WaitTime - w1.WaitTime > 0
           and w2.Waits - w1.Waits       > 0
    truncate table #monProcWait1
    insert #monProcWait1(SPID, KPID, WaitEventID, WaitTime, Waits)
        select SPID, KPID, WaitEventID, WaitTime, Waits 
          from #monProcWait2
    truncate table #monProcWait2

    insert #monDeviceIO2(Reads, APFReads, Writes, DevSemaphoreRequests, 
                         DevSemaphoreWaits, IOTime, LogicalName, PhysicalName)
        select Reads, APFReads, Writes, DevSemaphoreRequests, 
               DevSemaphoreWaits, IOTime, LogicalName, PhysicalName 
          from master..monDeviceIO
    insert mondb..monDeviceIO(Reads, APFReads, Writes, DevSemaphoreRequests, 
                              DevSemaphoreWaits, IOTime, 
                              LogicalName, PhysicalName, runDate)
        select d2.Reads - d1.Reads
             , d2.APFReads - d1.APFReads
             , d2.Writes - d1.Writes
             , d2.DevSemaphoreRequests - d1.DevSemaphoreRequests
             , d2.DevSemaphoreWaits - d1.DevSemaphoreWaits
             , d2.IOTime - d1.IOTime
             , d1.LogicalName, d1.PhysicalName
             , @dt
          from #monDeviceIO1 d1, #monDeviceIO2 d2
         where d1.PhysicalName = d2.PhysicalName
    truncate table #monDeviceIO1
    insert #monDeviceIO1 
        select Reads, APFReads, Writes, DevSemaphoreRequests, 
               DevSemaphoreWaits, IOTime, LogicalName, PhysicalName
          from #monDeviceIO2
    truncate table #monDeviceIO2

    insert #monNetIO2(PacketsSent, PacketsReceived, BytesSent, BytesReceived)
        select PacketsSent, PacketsReceived, BytesSent, BytesReceived 
          from master..monNetworkIO
    insert mondb..monNetworkIO(PacketsSent, PacketsReceived, BytesSent, 
                               BytesReceived, runDate)
        select n2.PacketsSent - n1.PacketsSent
             , n2.PacketsReceived - n1.PacketsReceived
             , n2.BytesSent - n1.BytesSent
             , n2.BytesReceived - n1.BytesReceived
             , @dt
          from #monNetIO1 n1
             , #monNetIO2 n2
     truncate table #monNetIO1
     insert #monNetIO1
         select PacketsSent, PacketsReceived, BytesSent, BytesReceived 
           from #monNetIO2
     truncate table #monNetIO2


     insert #monProcAct2(
              SPID,
              KPID,
              CPUTime,
              WaitTime,
              PhysicalReads    ,
              LogicalReads    ,
              PagesRead        ,
              PhysicalWrites    ,
              PagesWritten     ,
              MemUsageKB       ,
              LocksHeld        ,
              TableAccesses     ,
              IndexAccesses     ,
              TempDbObjects     ,
              ULCBytesWritten    ,
              ULCFlushes        ,
              ULCFlushFull      ,
              ULCMaxUsage       ,
              ULCCurrentUsage    ,
              Transactions       ,
              Commits            ,
              Rollbacks         )
        select SPID,
              KPID,
              CPUTime,
              WaitTime,
              PhysicalReads    ,
              LogicalReads    ,
              PagesRead        ,
              PhysicalWrites    ,
              PagesWritten     ,
              MemUsageKB       ,
              LocksHeld        ,
              TableAccesses     ,
              IndexAccesses     ,
              TempDbObjects     ,
              ULCBytesWritten    ,
              ULCFlushes        ,
              ULCFlushFull      ,
              ULCMaxUsage       ,
              ULCCurrentUsage    ,
              Transactions       ,
              Commits            ,
              Rollbacks 
         from master..monProcessActivity

    insert mondb..monProcessActivity(SPID,
              KPID,
              CPUTime,
              WaitTime,
              PhysicalReads    ,
              LogicalReads    ,
              PagesRead        ,
              PhysicalWrites    ,
              PagesWritten     ,
              MemUsageKB       ,
              LocksHeld        ,
              TableAccesses     ,
              IndexAccesses     ,
              TempDbObjects     ,
              ULCBytesWritten    ,
              ULCFlushes        ,
              ULCFlushFull      ,
              ULCMaxUsage       ,
              ULCCurrentUsage    ,
              Transactions       ,
              Commits            ,
              Rollbacks       ,
              Login  ,
              Application  ,
              ClientIP,
              runDate)
       select t2.SPID, t2.KPID
            , case when t2.CPUTime - t1.CPUTime < 0 then t2.CPUTime
                   else t2.CPUTime - t1.CPUTime
              end
            , t2.WaitTime - t1.WaitTime
            , t2.PhysicalReads - t1.PhysicalReads
            , t2.LogicalReads  - t1.LogicalReads
            , t2.PagesRead     - t1.PagesRead
            , t2.PhysicalWrites - t1.PhysicalWrites
            , t2.PagesWritten   - t1.PagesWritten
            , t2.MemUsageKB
            , t2.LocksHeld
            , t2.TableAccesses - t1.TableAccesses
            , t2.IndexAccesses - t1.IndexAccesses
            , t2.TempDbObjects - t1.TempDbObjects
            , t2.ULCBytesWritten - t1.ULCBytesWritten
            , t2.ULCFlushes - t1.ULCFlushes
            , t2.ULCFlushFull - t1.ULCFlushFull
            , t2.ULCMaxUsage
            , t2.ULCCurrentUsage
            , t2.Transactions - t1.Transactions
            , t2.Commits - t1.Commits
            , t2.Rollbacks - t1.Rollbacks
            , pl.Login
            , pl.Application
            , pl.ClientIP
            , @dt
         from #monProcAct1 t1
            , #monProcAct2 t2
            , master..monProcessLookup pl
        where t1.SPID = t2.SPID
          and t1.KPID = t2.KPID
          and pl.SPID = t2.SPID
          and pl.KPID = t2.KPID

    truncate table #monProcAct1
    insert #monProcAct1 select * from #monProcAct2
    truncate table #monProcAct2

    insert #monOpObj2 (
 DBID,
 ObjectID,
 LogicalReads,
 PhysicalReads,
 APFReads,
 PagesRead,
 PhysicalWrites,
 PagesWritten,
 UsedCount,
 RowsInserted,
 RowsDeleted,
 RowsUpdated,
 Operations,
 LockRequests,
 LockWaits )
     select 
 DBID,
 ObjectID,
 LogicalReads,
 PhysicalReads,
 APFReads,
 PagesRead,
 PhysicalWrites,
 PagesWritten,
 UsedCount,
 RowsInserted,
 RowsDeleted,
 RowsUpdated,
 Operations,
 LockRequests,
 LockWaits from master..monOpenObjectActivity where IndexID = 0

    insert mondb..monOpenObjectActivity ( DBID,
 ObjectID,
 LogicalReads,
 PhysicalReads,
 APFReads,
 PagesRead,
 PhysicalWrites,
 PagesWritten,
 UsedCount,
 RowsInserted,
 RowsDeleted,
 RowsUpdated,
 Operations,
 LockRequests,
 LockWaits,
 runDate )
    select  t1.DBID,
            t1.ObjectID,
            t2.LogicalReads - t1.LogicalReads,
            t2.PhysicalReads - t1.PhysicalReads,
            t2.APFReads - t1.APFReads,
            t2.PagesRead - t1.PagesRead,
            t2.PhysicalWrites - t1.PhysicalWrites,
            t2.PagesWritten - t1.PagesWritten,
            t2.UsedCount - t1.UsedCount,
            t2.RowsInserted - t1.RowsInserted,
            t2.RowsDeleted - t1.RowsDeleted,
            t2.RowsUpdated - t1.RowsUpdated,
            t2.Operations - t1.Operations,
            t2.LockRequests - t1.LockRequests,
            t2.LockWaits - t1.LockWaits,
            @dt
       from #monOpObj1 t1
          , #monOpObj2 t2
      where t2.Operations - t1.Operations > 0
        and t2.DBID = t1.DBID
        and t2.ObjectID = t1.ObjectID
        and object_name(t1.ObjectID, t1.DBID) not like '#%'
        and object_name(t1.ObjectID, t1.DBID) is not NULL

    truncate table #monOpObj1
    insert #monOpObj1 select * from #monOpObj2
    truncate table #monOpObj2


    --waitfor delay @delay
end

go

create proc sp__mda_collect
( @delay        varchar(20) = '00:01:00')
as
exec sp__mda_init
exec sp__mda_run_collect @delay

go


print "create indexes on mon tables"
go
create clustered index monDeviceIO_ix on monDeviceIO(servername, runDate, LogicalName)
go
create clustered index monEngine_ix on monEngine(servername, runDate, EngineNumber)
go
create clustered index monNetworkIO_ix on monNetworkIO(servername, runDate)
go
create clustered index monProcessActivity_ix 
    on monProcessActivity(servername, runDate, SPID)
go
create clustered index monProcessWaits_ix 
    on monProcessWaits(servername, runDate, SPID, WaitEventID)
go
create clustered index monOpenObjectActivity_ix
on monOpenObjectActivity(servername, runDate, DBID, ObjectID)
go

print "All done."
go
