/* $Id: histo_views.sql,v 1.1 2006/07/06 09:17:18 pepm01a Exp $ */

use mondb
go
if object_id('monProcessActivity_histo') != NULL
begin
    print "Dropping view monProcessActivity_histo"
    drop view monProcessActivity_histo
end
go

create view monProcessActivity_histo
( SPID
, KPID
, CPUTime
, WaitTime
, PhysicalReads
, LogicalReads
, PagesRead
, PhysicalWrites
, PagesWritten
, MemUsageKB
, LocksHeld
, TableAccesses
, IndexAccesses
, TempDbObjects
, ULCBytesWritten
, ULCFlushes
, ULCFlushFull
, ULCMaxUsage
, ULCCurrentUsage
, Transactions
, Commits
, Rollbacks
, Login
, Application
, ClientIP
, runDate
, servername
)
as
select SPID, KPID
     , sum(CPUTime)
     , sum(WaitTime)
     , sum(PhysicalReads)
     , sum(LogicalReads)
     , sum(PagesRead)
     , sum(PhysicalWrites)
     , sum(PagesWritten)
     , max(MemUsageKB)
     , max(LocksHeld)
     , sum(TableAccesses)
     , sum(IndexAccesses)
     , sum(TempDbObjects)
     , sum(ULCBytesWritten)
     , sum(ULCFlushes)
     , sum(ULCFlushFull)
     , max(ULCMaxUsage)
     , avg(ULCCurrentUsage)
     , sum(Transactions)
     , sum(Commits)
     , sum(Rollbacks)
     , max(Login)
     , max(Application)
     , max(ClientIP)
     , convert(datetime, convert(char(12), runDate, 112) + 
               convert(varchar(2), datepart(hh, runDate)) + ":00")
     , max(servername)
  from monProcessActivity
 where servername = (select srvnetname from master..sysservers where srvname = 'loopback')
   and runDate >= convert(varchar, dateadd(dd, -1, getdate()), 112)
   and runDate <  convert(varchar, getdate(), 112)
--   and runDate between convert(varchar, dateadd(dd, -1, getdate()), 112)
--                   and convert(varchar, getdate(), 112)
 group by SPID, KPID, 
       convert(char(12), runDate, 112) + 
       convert(varchar(2), datepart(hh, runDate)) + ":00"
go

if object_id('monProcessActivity_histo') != NULL
begin
    print "Created view monProcessActivity_histo"
    grant select on monProcessActivity_histo to public
end
go

if object_id('monEngine_histo') != NULL
begin
    print "Dropping view monEngine_histo"
    drop view monEngine_histo
end
go

create view monEngine_histo
( EngineNumber
, CPUTime
, SystemCPUTime
, UserCPUTime
, IdleCPUTime
, ContextSwitches
, runDate
, servername
)
as
select EngineNumber
     , sum(CPUTime)
     , sum(SystemCPUTime)
     , sum(UserCPUTime)
     , sum(IdleCPUTime)
     , sum(ContextSwitches)
     , convert(datetime, convert(char(12), runDate, 112) + 
               convert(varchar(2), datepart(hh, runDate)) + ":00")
     , max(servername)
  from monEngine 
 where servername = (select srvnetname from master..sysservers where srvname = 'loopback')
   and runDate >= convert(varchar, dateadd(dd, -1, getdate()), 112)
   and runDate <  convert(varchar, getdate(), 112)
 group by EngineNumber,
       convert(char(12), runDate, 112) + 
       convert(varchar(2), datepart(hh, runDate)) + ":00"

go

if object_id('monEngine_histo') != NULL
begin
    print "Created view monEngine_histo"
    grant select on monEngine_histo to public
end
go

if object_id('monNetworkIO_histo') != NULL
begin
    print "Dropping view monNetworkIO_histo"
    drop view monNetworkIO_histo
end
go

create view monNetworkIO_histo
( PacketsSent
, PacketsReceived
, BytesSent
, BytesReceived
, runDate
, servername
)
as
select sum(PacketsSent)
     , sum(PacketsReceived)
     , sum(BytesSent)
     , sum(BytesReceived)
     , convert(datetime, convert(char(12), runDate, 112) + 
               convert(varchar(2), datepart(hh, runDate)) + ":00")
     , max(servername)
  from monNetworkIO 
 where servername = (select srvnetname from master..sysservers where srvname = 'loopback')
   and runDate >= convert(varchar, dateadd(dd, -1, getdate()), 112)
   and runDate <  convert(varchar, getdate(), 112)
--   and runDate between convert(varchar, dateadd(dd, -1, getdate()), 112)
--                   and convert(varchar, getdate(), 112)
 group by convert(char(12), runDate, 112) + 
       convert(varchar(2), datepart(hh, runDate)) + ":00"

if object_id('monNetworkIO_histo') != NULL
begin
    print "Created view monNetworkIO_histo"
    grant select on monNetworkIO_histo to public
end
go

if object_id('monDeviceIO_histo') != NULL
begin
    print "Dropping view monDeviceIO_histo"
    drop view monDeviceIO_histo
end
go
create view monDeviceIO_histo
( Reads
, APFReads
, Writes
, DevSemaphoreRequests
, DevSemaphoreWaits
, IOTime
, LogicalName
, PhysicalName
, runDate
, servername
)
as
select sum(Reads)
     , sum(APFReads)
     , sum(Writes)
     , sum(DevSemaphoreRequests)
     , sum(DevSemaphoreWaits)
     , sum(IOTime)
     , LogicalName
     , max(PhysicalName)
     , convert(datetime, convert(char(12), runDate, 112) + 
               convert(varchar(2), datepart(hh, runDate)) + ":00")
     , max(servername)
  from monDeviceIO 
 where servername = (select srvnetname from master..sysservers where srvname = 'loopback')
   and runDate >= convert(varchar, dateadd(dd, -1, getdate()), 112)
   and runDate <  convert(varchar, getdate(), 112)
--   and runDate between convert(varchar, dateadd(dd, -1, getdate()), 112)
--                   and convert(varchar, getdate(), 112)
  group by LogicalName
     , convert(char(12), runDate, 112) + 
       convert(varchar(2), datepart(hh, runDate)) + ":00"
go
if object_id('monDeviceIO_histo') != NULL
begin
    print "Created view monDeviceIO_histo"
    grant select on monDeviceIO_histo to public
end
go

if object_id('monProcessWaits_histo') != NULL
begin
    print "Dropping view monProcessWaits_histo"
    drop view monProcessWaits_histo
end
go

create view monProcessWaits_histo
( SPID
, KPID
, WaitEventID
, WaitTime
, Waits
, runDate
, servername
)
as
select SPID
     , KPID
     , WaitEventID
     , sum(WaitTime)
     , sum(Waits)
     , convert(datetime, convert(char(12), runDate, 112) + 
               convert(varchar(2), datepart(hh, runDate)) + ":00")
     , max(servername)
  from monProcessWaits 
 where servername = (select srvnetname from master..sysservers where srvname = 'loopback')
   and runDate >= convert(varchar, dateadd(dd, -1, getdate()), 112)
   and runDate <  convert(varchar, getdate(), 112)
--   and runDate between convert(varchar, dateadd(dd, -1, getdate()), 112)
--                   and convert(varchar, getdate(), 112)
   and WaitEventID not in (19, 57, 61, 85, 176, 250, 260, 266)
 group by SPID, KPID, WaitEventID
     , convert(char(12), runDate, 112) + 
       convert(varchar(2), datepart(hh, runDate)) + ":00"
 having sum(WaitTime) > 0 or sum(Waits) > 0
go

if object_id('monProcessWaits_histo') != NULL
begin
    print "Created view monProcessWaits_histo"
    grant select on monProcessWaits_histo to public
end
go

