#!/usr/bin/perl -w
#
# $Id: pull-histo.pl,v 1.1 2006/07/19 06:46:24 pepm01a Exp $

use strict;
use Getopt::Long;

$ENV{SYBASE}='/pvar/sybase/15.0';
$ENV{PATH}="$ENV{SYBASE}/OCS-15_0/bin:$ENV{PATH}";



my %opts;

GetOptions(\%opts, 'server=s', 'uid=s', 'pwd=s');

die "Usage: $0 --server <name> [--uid <name>] [--pwd <string>]" 
    unless $opts{server};

my @tables = qw(monDeviceIO monEngine monNetworkIO monProcessActivity
		monProcessWaits);

my %source = ( server => $opts{server},
	       pwd    => $opts{pwd},
	       uid    => $opts{uid},
	       db     => 'mondb'
	       );
my %dest = ( server => 'DBA_SQL',
	     pwd => 'guess_me',
	     uid => 'sa',
	     db => 'histo');

my $tmppath = "/var/tmp";

$ENV{LANG} = '';
$ENV{LC_ALL} = '';

foreach my $tab (@tables) {
    system("bcp $source{db}..${tab}_histo$wk out $tmppath/$tab.$source{server}.bcp -U$source{uid} -P$source{pwd} -S$source{server} -c");
    if($? != 0) {
	warn("bcp out for $tab failed");
    }
    # The MDA counters will overflow (i.e. wrap) when they go beyond 2^31,
    # and the sp__mda_collect proc doesn't currently handle this.
    # So check the data in the bcp files and fix any large negative values.
    fixOverflow("$tmppath/$tab.$source{server}.bcp");
    system("bcp $dest{db}..${tab} in $tmppath/$tab.$source{server}.bcp -U$dest{uid} -P$dest{pwd} -S$dest{server} -c -e $tmppath/$tab.err");
    if($? != 0) {
	warn("bcp in for $tab failed");
    }
}


sub fixOverflow {
    my $file = shift;

    rename($file, "$file.tmp") || die "Can't rename $file: $!";
    open(IN, "$file.tmp") || die "Can't open $file.tmp: $!";
    open(OUT, ">$file") || die "Can't open $file: $!";

    while(<IN>) {
	chomp;
	# Change any large negative numbers by their 2s complement.
	# smaller negative numbers are converted to 0 to avoid possible issues.
	s/\s(-\d+)/"\t" . ($1 < -1000000000 ? 2147483648 + $1 : 0)/eg;
	print OUT "$_\n";
    }
}
