-- $Id: sp__allusage.sql,v 1.1 2006/07/19 06:46:24 pepm01a Exp $
-- Original code from Mich Talebzadeh, with input from Michael Peppler
use sybsystemprocs
go
IF EXISTS(SELECT 1 FROM sysobjects WHERE type = 'P' AND name = 'sp__allusage')
BEGIN
  DROP PROCEDURE sp__allusage
END
go
create procedure sp__allusage
as
begin
set nocount on
/* This code originated with Mich Talebzadeh */
set nocount on

-- Use the db you want to analyse, and then run this script.
--
-- First of all work out the table rows and sizes (reserved)
--
select  
        "Owner" = user_name(o.uid),
	TableName = o.name,
        IndexName = i.name,
        low = d.low,
        rowtotal = rowcnt(i.doampg),
        reserved = (reserved_pgs(i.id, i.doampg) + reserved_pgs(i.id, i.ioampg)) * (low / 1024)
into #t1
from sysobjects o, sysindexes i, master.dbo.spt_values d
where
      o.type = 'U'
      and i.id = o.id
      and d.number = 1
      and d.type = "E"

select distinct
       Owner,
       TableName,
       RowTotal = convert(char(11), sum(rowtotal)),
       Reserved =  sum(reserved)
into #table_size
from
     #t1
group by Owner,
         TableName
--
-- Identify dormant tables
--
  SELECT
  "Owner" = user_name(o.uid),
  "TableName" = o.name,
  "LogicalReads" = m.LogicalReads,
  "LockRequests" = m.LockRequests,
  "Operations" = m.Operations,
  "Selected" = m.OptSelectCount,
  "WhenLastSelected" = m.LastOptSelectDate,
  "Used" = m.UsedCount,
  "WhenLastUsed" = m.LastUsedDate,
  "Inserted" = m.RowsInserted,
  "Updated" = m.RowsUpdated,
  "Deleted" = m.RowsDeleted
  INTO
      #dormant
  from
       sysobjects o,
       master..monOpenObjectActivity m
  where
       object_name(m.ObjectID, m.DBID) = o.name
       and o.type = 'U'
       and o.id = m.ObjectID
       and m.IndexID = 0
       and m.DBID = db_id()
       and object_name(m.ObjectID, m.DBID) not like 'sa_%'
       and object_name(m.ObjectID, m.DBID) not like '%__sa%'
       and object_name(m.ObjectID, m.DBID) not like 'rs_%'
       and m.RowsInserted = 0
       and m.RowsUpdated = 0
       and m.RowsDeleted = 0
  PRINT ""
  PRINT "Displaying dormant tables with no DML activity"
  PRINT ""
  SELECT
       "TableName" = substring(t.Owner+"."+t.TableName, 1, 35),
       "Rows" = convert(numeric(10,0),s.RowTotal),
       "Size/KB" = convert(numeric(10,0),s.Reserved),
       "LogicalReads" = t.LogicalReads,
       "LockRequests" = t.LockRequests
  FROM
       #dormant t,
       #table_size s
  WHERE
       t.Owner = s.Owner
       and t.TableName = s.TableName
       and t.WhenLastUsed is NULL
       and not exists (select 1 from master..monOpenObjectActivity m
                  where object_name(m.ObjectID, m.DBID) = t.TableName
                    and object_name(m.ObjectID, m.DBID) = s.TableName
                    and m.DBID = db_id()
                    and m.IndexID > 0
                    and m.LastUsedDate is not NULL)
  ORDER BY
        t.Owner,
        t.TableName
--
-- Identify high activity tables
--
  SELECT
  "Owner" = user_name(o.uid),
  "TableName" = o.name,
  "LogicalReads" = m.LogicalReads,
  "LockRequests" = m.LockRequests,
  "Operations" = m.Operations,
  "Selected" = m.OptSelectCount,
  "WhenLastSelected" = m.LastOptSelectDate,
  "Used" = m.UsedCount,
  "WhenLastUsed" = m.LastUsedDate,
  "Inserted" = m.RowsInserted,
  "Updated" = m.RowsUpdated,
  "Deleted" = m.RowsDeleted
  INTO
      #temp
  from
       sysobjects o,
       master..monOpenObjectActivity m
  where
       object_name(m.ObjectID, m.DBID) = o.name
       and o.type = 'U'
       and o.id = m.ObjectID
       and m.IndexID = 0
       and m.DBID = db_id()
       and object_name(m.ObjectID, m.DBID) not like 'sa_%'
       and object_name(m.ObjectID, m.DBID) not like '%__sa%'
       and object_name(m.ObjectID, m.DBID) not like 'rs_%'
       and (m.RowsInserted > 0 or m.RowsUpdated > 0 or  m.RowsDeleted > 0)
    --   and m.LastOptSelectDate is NOT NULL
    --   and m.LastUsedDate is NOT NULL

  PRINT ""
  PRINT "Displaying high activity tables"
  PRINT ""
  SELECT
       "TableName" = substring(t.Owner+"."+t.TableName, 1, 20),
       "Rows" = convert(numeric(10,0),s.RowTotal),
       "Size/KB" = convert(numeric(10,0),s.Reserved),
        t.Inserted,
        t.Updated,
        t.Deleted,
        t.LockRequests
  FROM
       #temp t,
       #table_size s
  WHERE
       t.Owner = s.Owner
       and t.TableName = s.TableName
  --     and t.WhenLastSelected is NOT NULL
  --     and t.WhenLastUsed is NOT NULL
  ORDER BY
        t.Owner,
        t.TableName
--
SELECT
"TableName" = object_name(m.ObjectID, m.DBID),
"IndexName" = i.name,
"Selected" = m.OptSelectCount,
"WhenLastSelected" = m.LastOptSelectDate,
"Used" = m.UsedCount,
"WhenLastUsed" = m.LastUsedDate
into #used
from master..monOpenObjectActivity m,
     sysindexes i
where
    m.IndexID > 0
and m.IndexID <> 255 -- ignore text, image data chain
and m.IndexID = i.indid
and m.ObjectID = i.id
and m.DBID = db_id()
print ""
if exists(select 1 from #used where Selected = 0 and Used = 0)
begin
  print ""
  print 'Indexes never selected or used by the optimizer'
  print ""
  select  "TableName" = substring(u.TableName, 1, 20),
          u.IndexName,
          "IndexSize/KB" = i.reserved,
          u.Selected,
          u.Used
  from  #used u,
        #t1 i
  where u.TableName = i.TableName
        and u.IndexName = i.IndexName
        and u.Selected = 0 and u.Used = 0
  order by u.TableName
end
if exists(select 1 from #used where Selected > 0 and Used = 0)
begin
  print ""
  print 'Indexes selected by the optimizer but never used in query'
  print ""
  select  "TableName" = substring(u.TableName, 1, 20),
        u.IndexName,
        "IndexSize/KB" = i.reserved,
        u.Selected,
        "When Last selected" = u.WhenLastSelected
  from  #used u,
        #t1 i
  where u.TableName = i.TableName
        and u.IndexName = i.IndexName
        and u.Selected > 0 and u.Used = 0
  order by u.TableName
end
if exists(select 1 from #used where Selected = 0 and Used > 0)
begin
  print ""
  print 'Indexes Used by the optimizer but never selected'
  print ""
  select  "TableName" = substring(u.TableName, 1, 20),
        u.IndexName,
        "IndexSize/KB" = i.reserved,
        u.Selected
  from  #used u,
        #t1 i
  where u.TableName = i.TableName
        and u.IndexName = i.IndexName
        and u.Selected = 0 and u.Used > 0
  order by u.TableName
end
select  TableName,
        IndexName,
        Selected,
        Used
into    #clean
from    #used
where   Used > 0
and	Selected > 0
--
select  TableName,
        IndexName,
        Selected,
        Used,
        "Selected_over_sum_selected" = convert(numeric(10,4),Selected*1.0/sum(Selected)*1.0),
        "Used_over_sum_used" = convert(numeric(10,4),Used*1.0/sum(Used)*1.0),
        "Used_over_selected" = convert(numeric(10,4),Used*1.0/Selected*1.0)
into #results
from #clean
group by TableName
if exists (select 1 from #results)
begin
  print ""
  print 'Index usage analysis'
  print ""
  print "R1 = Selected/SUM(Selected)"
  print "R2 = Used/SUM(Used)"
  print "R3 = Used/Selected"
  print ""
  select "TableName" = substring(r.TableName, 1, 15),
        "IndexName" = substring(r.IndexName, 1, 20),
        "Size/KB" = i.reserved,
        r.Selected,
        r.Used,
        "R1" = str(r.Selected_over_sum_selected, 9, 3),
        "R2" = str(r.Used_over_sum_used, 9, 3),
        "R3" = str(r.Used_over_selected, 9, 3)
  from  #results r,
        #t1 i
  where 
        r.TableName = i.TableName
        and  r.IndexName = i.IndexName
        and r.Used_over_sum_used < 1.0
  order by r.TableName, r.Used_over_sum_used desc
end


-- Identify tables accessed via a table scan
  SELECT
  "Owner" = user_name(o.uid),
  "TableName" = o.name,
  "LogicalReads" = m.LogicalReads,
  "PagesRead" = m.PagesRead,
  "WhenLastSelected" = m.LastOptSelectDate,
  "Used" = m.UsedCount
  INTO
      #tabscan
  from
       sysobjects o,
       master..monOpenObjectActivity m
  where
       o.type = 'U'
       and o.id = m.ObjectID
       and m.IndexID = 0
       and m.DBID = db_id()
       and object_name(m.ObjectID, m.DBID) not like 'sa_%'
       and object_name(m.ObjectID, m.DBID) not like '%__sa%'
       and object_name(m.ObjectID, m.DBID) not like 'rs_%'
       and m.LastUsedDate is not NULL
       and m.UsedCount > 0
print ""
print 'Tables accessed via Table Scans'
print ""
  SELECT
       "TableName" = substring(t.Owner+"."+t.TableName, 1, 25),
       "Rows" = convert(numeric(8,0),s.RowTotal),
       "Size/KB" = convert(numeric(8,0),s.Reserved),
       "LogicalReads" = t.LogicalReads,
       "PagesRead" = t.PagesRead,
       "Used" = str(t.Used, 5, 0),
       t.WhenLastSelected
  FROM
       #tabscan t,
       #table_size s
  WHERE
       t.Owner = s.Owner
       and t.TableName = s.TableName
  ORDER BY
        t.Owner,
        t.TableName


end
go
grant exec on sp__allusage to public
go

