-- $Id: sp__cache.sql,v 1.1 2006/07/19 06:46:24 pepm01a Exp $
-- 
-- sp__cache
-- Created by Mich Talebzadeh and Michael Peppler

use sybsystemprocs
go
IF EXISTS(SELECT 1 FROM sysobjects WHERE type = 'P' AND name =
'sp__cache')
BEGIN
DROP PROCEDURE sp__cache
END
go
create procedure sp__cache with recompile
as
begin
set nocount on
declare @time datetime
select @time = getdate()
select CacheName, 
       "AllocatedMB" = convert(char(6),sum(AllocatedKB)/1024)
  into #cachesize
  from master..monCachePool
 group by CacheName
print ""
print "Cache status at %1!. Server up since %2!", @time, @@boottime
print ""
print "Notes -- During I/O completion processing when a physical I/O
needs to be placed into a buffer in a cache's pool, if the buffer at
the LRU is still dirty (changed but not persisted to disk), the engine
STALLS waiting for it. The engine cannot do anything until the I/O can
be processed by storing it in that buffer.  This is a bad situation from
a performance perspective."
print ""
print "Watch for 'STALLS/Buffers grabbed dirty' column below"
print ""
select "Cache Name" = substring(m.CacheName+"("+t.AllocatedMB+"MB)",1,30),
       "Searches" = m.CacheSearches,
       "Logical Reads" = m.LogicalReads,
       "Phys. Reads" = m.PhysicalReads,
       "Phys. Writes" = m.PhysicalWrites,
       "STALLS" = m.Stalls
  from master..monDataCache m,
       #cachesize t
 where m.CacheName = t.CacheName
 order by m.CacheID
print ""
select "Cache Name" = substring(CacheName, 1, 15),
       "Pool Type" = convert(tinyint, IOBufferSize/1024),
       "Pool size/MB" = AllocatedKB/1024,
       "Used/MB" = (PagesTouched * (@@maxpagesize/1024))/1024,
       "Pages read" = PagesRead,
       "Moved to MRU" = BuffersToMRU ,
       "fetch/discard" =  BuffersToLRU,
       "Stalls" = Stalls
  from master..monCachePool
 order by CacheID, IOBufferSize
end
go
grant exec on sp__cache to public
go
